/**
 **	Query.C
 **
 **	Published / author: 2001-02-15 / grymse@telia.com
 **/

/*
Copyright (C) 2001  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql/mysql.h>
#ifdef THREAD_SAFE
#ifdef USE_SDL
#include <SDL.h>
#else
#include <pthread.h>
#endif // USE_SDL
#endif // THREAD_SAFE

#include "Database.h"
#include "Query.h"


Query::Query(Database *dbin)
:m_db(*dbin)
,odb(dbin -> grabdb())
,res(NULL)
,row(NULL)
{
}


Query::Query(Database& dbin) : m_db(dbin),odb(dbin.grabdb()),res(NULL),row(NULL)
{
}


Query::Query(Database *dbin,char *sql)
:m_db(*dbin)
,odb(dbin -> grabdb())
,res(NULL)
,row(NULL)
{
	if (!execute(sql))
	{
		fprintf(stderr,"query failed: '%s'\n",sql);
	}
}


Query::Query(Database& dbin,const std::string& sql) : m_db(dbin),odb(dbin.grabdb()),res(NULL),row(NULL)
{
	if (!execute(sql))
	{
		fprintf(stderr,"query failed: '%s'\n",sql.c_str());
	}
}


// destructor

Query::~Query()
{
//printf("~Query()\n");
	if (res)
		mysql_free_result(res);
	if (odb)
		m_db.freedb(odb);
}


short Query::execute(const std::string& sql)
{		// query, no result
	if (odb && res)
		fprintf(stderr,"query busy\n");
	if (odb && !res)
	{
		if (mysql_query(&odb -> mysql,sql.c_str()))
			fprintf(stderr,"query failed: '%s'\n",sql.c_str());
		else
			return 1;
	}
	return 0;
}



// methods using db specific api calls

MYSQL_RES *Query::get_result(const std::string& sql)
{	// query, result
	if (odb && res)
		fprintf(stderr,"query busy\n");
	if (odb && !res)
	{
		if (execute(sql))
			res = mysql_store_result(&odb -> mysql);
		else
		{
			fprintf(stderr,"query failed: '%s'\n",sql.c_str());
//			if (_debug)
//				printf("\nquery failed: '%s'<br>\n",sql);
		}
	}
	return res;
}


void Query::free_result()
{
	if (odb && res)
	{
		mysql_free_result(res);
		res = NULL;
		row = NULL;
	}
}


MYSQL_ROW Query::fetch_row()
{
	rowcount = 0;
	return odb && res ? row = mysql_fetch_row(res) : NULL;
}


long Query::insert_id()
{
	if (odb)
		return mysql_insert_id(&odb -> mysql);
	else
		return -1;
}


long Query::num_rows()
{
	return odb && res ? mysql_num_rows(res) : 0;
}


// data retreival methods

const char *Query::getstr(int x)
{
	if (odb && res && row)
		return row[x] ? row[x] : "";
	else
		return NULL;
}


const char *Query::getstr()
{
	return getstr(rowcount++);
}


double Query::getnum(int x)
{
	return odb && res && row && row[x] ? atof(row[x]) : 0;
}


long Query::getval(int x)
{
	return odb && res && row && row[x] ? atol(row[x]) : 0;
}


double Query::getnum()
{
	return getnum(rowcount++);
}


long Query::getval()
{
	return getval(rowcount++);
}


double Query::get_num(const std::string& sql)
{
	double l = 0;
	if (get_result(sql))
	{
		if (fetch_row())
			l = getnum();
		free_result();
	}
	return l;
}


long Query::get_count(const std::string& sql)
{
	long l = 0;
	if (get_result(sql))
	{
		if (fetch_row())
			l = getval();
		free_result();
	}
	return l;
}


const char *Query::get_string(const std::string& sql)
{
static	char slask[30000]; // %! this will break eventually, sorry
	*slask = 0;
	if (get_result(sql))
	{
		if (fetch_row())
		{
			strncpy(slask,getstr(),29999);
			slask[29999] = 0;
		}
		free_result();
	}
	return slask; // %! changed from 1.0 which didn't return NULL on failed query
}


/*
void Query::debug(short val)
{ 
	_debug = val; 
}


void Query::syslog(short val)
{ 
	_syslog = val; 
}
*/


//MYSQL_FIELD *	STDCALL mysql_fetch_field(MYSQL_RES *result);


MYSQL_FIELD *Query::fetch_field()
{
	return odb && res ? mysql_fetch_field(res) : NULL;
}


const char *Query::fetch_fieldname()
{
	MYSQL_FIELD *field = odb && res ? mysql_fetch_field(res) : NULL;
	return field ? field -> name : "";
}


/*
void Query::safestr(char *s) 
{
	char *s2 = new char[strlen(s) * 2 + 2];
	int i,j = 0;
	*s2 = 0;
	for (i = 0; i < (int)strlen(s); i++)
		switch (s[i])
		{
		case '\'':
		case '\\':
		case 34:
			s2[j++] = '\\';
		default:
			s2[j++] = s[i];
		}
	s2[j] = 0;
	strcpy(s,s2);
}


void Query::unsafestr(char *s) 
{
	int i;
	for (i = 0; i < (int)strlen(s); i++)
		if (s[i] == '\\')
			memmove(s + i,s + i + 1,strlen(s + i) + 1);
}


char *Query::xmlsafestr(char *s) 
{
static	char str[1999];
	int i;
	*str = 0;
	for (i = 0; i < (int)strlen(s); i++)
	{
		switch (s[i])
		{
		case '&':
			strcat(str,"&amp;");
			break;
		case '<':
			strcat(str,"&lt;");
			break;
		case '>':
			strcat(str,"&gt;");
			break;
		case '"':
			strcat(str,"&quot;");
			break;
		case '\'':
			strcat(str,"&apos;");
			break;
		default:
			sprintf(str + strlen(str),"%c",s[i]);
		}
	}
	return str;
}
*/


std::string Query::safestr(const std::string& str)
{
	std::string str2;
	for (size_t i = 0; i < str.size(); i++)
	{
		switch (str[i])
		{
		case '\'':
		case '\\':
		case 34:
			str2 += '\\';
		default:
			str2 += str[i];
		}
	}
	return str2;
}


std::string Query::unsafestr(const std::string& str)
{
	std::string str2;
	for (size_t i = 0; i < str.size(); i++)
	{
		if (str[i] == '\\')
		{
			i++;
		}
		if (i < str.size())
		{
			str2 += str[i];
		}
	}
	return str2;
}


std::string Query::xmlsafestr(const std::string& str)
{
	std::string str2;
	for (size_t i = 0; i < str.size(); i++)
	{
		switch (str[i])
		{
		case '&':
			str2 += "&amp;";
			break;
		case '<':
			str2 += "&lt;";
			break;
		case '>':
			str2 += "&gt;";
			break;
		case '"':
			str2 += "&quot;";
			break;
		case '\'':
			str2 += "&apos;";
			break;
		default:
			str2 += str[i];
		}
	}
	return str2;
}


